@@ -1,6 +1,6 @@
 # GNU Solfege - free ear training software
 # vim: set fileencoding=utf-8 :
-# Copyright (C) 2000, 2001, 2002, 2003, 2004, 2006, 2007, 2008  Tom Cato Amundsen
+# Copyright (C) 2000, 2001, 2002, 2003, 2004, 2006, 2007, 2008, 2011  Tom Cato Amundsen
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,18 +16,23 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 from __future__ import absolute_import
+from __future__ import division
 
 import os
 import re
+import subprocess
 import sys
 import time
+import traceback
 
 import gobject
 import gtk
 import pango
 
 from solfege import cfg
+from solfege import osutils
 from solfege import soundcard
+from solfege import utils
 
 import solfege
 
@@ -160,7 +165,7 @@ class nCheckButton(gtk.CheckButton, cfg.ConfigUtils):
         self.set_bool(self.m_name, self.get_active())
         if self.m_callback:
             self.m_callback(_o)
- 
+
 def RadioButton(group, label, callback=None):
     rdb = gtk.RadioButton(group, label)
     if callback:
@@ -223,17 +228,22 @@ class PercussionNameButton(gtk.Button, cfg.ConfigUtils):
     def entry_changed(self, widget, value):
         self.set_int(self.m_name, value + soundcard.first_percussion_int_value)
         self.set_label(soundcard.percussion_names[value])
-
+        t = utils.new_percussion_track()
+        t.note(8, value + soundcard.first_percussion_int_value)
+        t.note(16, value + soundcard.first_percussion_int_value)
+        t.note(16, value + soundcard.first_percussion_int_value)
+        t.note(4, value + soundcard.first_percussion_int_value)
+        soundcard.synth.play_track(t)
 
 class FlashBar(gtk.Frame):
     def __init__(self):
         gtk.Frame.__init__(self)
         self.set_shadow_type(gtk.SHADOW_IN)
-        #FIXME different gtk themes can make these values wrong
-        self.set_size_request(-1, 40)
         self.__stack = []
         self.__label = HarmonicProgressionLabel('')
-        self.add(self.__label)
+        self.__content = gtk.HBox()
+        self.__content.show()
+        self.add(self.__content)
         self.__timeout = None
         # The allocated size
         self.m_sx, self.m_sy = 0, 0
@@ -244,41 +254,57 @@ class FlashBar(gtk.Frame):
         show all the strings in stringlist.
         """
         for s in stringlist:
-            self.__label.set_text(s)
-            x, y =  self.__label.size_request()
-            self.m_sx = max(x, self.m_sx)
-            self.m_sy = max(y, self.m_sy)
-        self.__label.set_text("")
-        self.__label.set_size_request(self.m_sx, self.m_sy)
+            self.display(s)
+        self.empty()
     def delayed_flash(self, milliseconds, msg):
         gobject.timeout_add(milliseconds, lambda: self.flash(msg))
-    def flash(self, txt):
+    def empty(self):
+        for child in self.__content.get_children():
+            child.destroy()
+    def flash(self, txt, **kwargs):
         """Display a message that is automatically removed after some time.
         If we flash a new message before the old flashed message are removed,
         we old flashed message are removed.
         """
         if self.__timeout:
             gobject.source_remove(self.__timeout)
-        self.__label.set_size_request(-1, -1)
-        self.__label.set_text(txt)
-        sx, sy = self.__label.size_request()
-        self.m_sx = max(sx, self.m_sx)
-        self.m_sy = max(sy, self.m_sy)
-        self.__label.set_size_request(self.m_sx, self.m_sy)
+        self.display(txt, **kwargs)
         def f(self=self):
             self.__timeout = None
             if self.__stack:
-                self.__label.set_text(self.__stack[-1])
+                self.display(self.__stack[-1][0], **self.__stack[-1][1])
             else:
-                self.__label.set_text('')
+                self.empty()
         self.__timeout = gobject.timeout_add(2000, f)
-    def push(self, txt):
+    def push(self, txt, **kwargs):
         # stop any flashing before we push
+        self.display(txt, **kwargs)
+        self.__stack.append((txt, kwargs))
+    def display(self, txt, **kwargs):
+        self.empty()
+        r = re.compile("(\{\w+\})") # Unicode??
+        self.set_size_request(-1, -1)
+        for child in r.split(txt):
+            m = r.match(child)
+            if m:
+                varname = child[1:][:-1]
+                from solfege import lessonfilegui
+                w = lessonfilegui.new_labelobject(kwargs[varname])
+                w.show()
+                self.__content.pack_start(w, False)
+            else:
+                if child:
+                    label = gtk.Label(child)
+                    label.set_name("FlashBarLabel")
+                    self.__content.pack_start(label, False)
+                    label.show()
+        sx, sy = self.size_request()
+        self.m_sx = max(sx, self.m_sx)
+        self.m_sy = max(sy, self.m_sy)
+        self.set_size_request(self.m_sx, self.m_sy)
         if self.__timeout:
             gobject.source_remove(self.__timeout)
             self.__timeout = None
-        self.__stack.append(txt)
-        self.__label.set_text(txt)
     def pop(self):
         """If a message is being flashed right now, that flashing is
         not affected, but the message below the flashed message is removed.
@@ -287,19 +313,20 @@ class FlashBar(gtk.Frame):
             self.__stack.pop()
         if not self.__timeout:
             if self.__stack:
-                self.__label.set_text(self.__stack[-1])
+                self.display(self.__stack[-1][0],
+                           **self.__stack[-1][1])
             else:
-                self.__label.set_text('')
+                self.empty()
     def clear(self):
         self.__stack = []
         if not self.__timeout:
-            self.__label.set_text('')
-    def set(self, txt):
+            self.empty()
+    def set(self, txt, **kwargs):
         """
         Empty the stack of messages and display the string in txt.
         """
-        self.__stack = [txt]
-        self.__label.set_text(txt)
+        self.__stack = [(txt, kwargs)]
+        self.display(txt, **kwargs)
 
 class hig(object):
     SPACE_SMALL= 6
@@ -494,6 +521,16 @@ def dialog_ok(text, parent=None, secondary_text=None, msgtype=gtk.MESSAGE_INFO):
     m.destroy()
     return ret
 
+def dialog_delete(text, parent, secondary_text=None):
+    m = gtk.MessageDialog(parent, gtk.DIALOG_MODAL, gtk.MESSAGE_QUESTION, gtk.BUTTONS_NONE, text)
+    m.add_buttons(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_DELETE, gtk.RESPONSE_ACCEPT)
+    if secondary_text:
+        m.format_secondary_text(secondary_text)
+    ret = m.run()
+    m.destroy()
+    return ret == gtk.RESPONSE_ACCEPT
+    return ret
+
 class NewLineBox(gtk.VBox):
     def __init__(self):
         gtk.VBox.__init__(self)
@@ -710,17 +747,20 @@ class EditorDialogBase(object):
                 # to do handle string to int conversion in
                 # PractiseSheet.parse_file. Just let int() raise ValueException
                 # if the string in not an integer.
-                solfege.win.display_exception_message(e)
+                display_exception_message(e)
         dialog.destroy()
-    def on_save_as(self, widget):
-        """
-        Return True if the file was saved, False if not.
-        """
+    def get_save_as_dialog(self):
         dialog = gtk.FileChooserDialog(_("Save As..."), self,
             gtk.FILE_CHOOSER_ACTION_SAVE,
             (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
              gtk.STOCK_OK, gtk.RESPONSE_OK))
         dialog.set_default_response(gtk.RESPONSE_OK)
+        return dialog
+    def on_save_as(self, widget):
+        """
+        Return True if the file was saved, False if not.
+        """
+        dialog = self.get_save_as_dialog()
         if not os.path.exists(self.savedir):
             try:
                 os.mkdir(self.savedir)
@@ -803,34 +843,14 @@ class EditorDialogBase(object):
         return ret
 
 
-class LogWindow(gtk.Window):
-    def __init__(self, parent):
-        gtk.Window.__init__(self)
-        self.set_transient_for(parent)
-        self.set_destroy_with_parent(True)
-        self.set_modal(True)
-        vbox = gtk.VBox()
-        vbox.set_spacing(4)
-        self.add(vbox)
-        self.set_default_size(630, 400)
+class LogWidget(gtk.ScrolledWindow):
+    def __init__(self):
+        gtk.ScrolledWindow.__init__(self)
         self.g_textbuffer = gtk.TextBuffer()
         self.g_textbuffer.create_tag('h1', weight=pango.WEIGHT_BOLD,
             size=16*pango.SCALE)
         self.g_textview = gtk.TextView(buffer=self.g_textbuffer)
-        self.g_scrolledwindow = gtk.ScrolledWindow()
-        vbox.pack_start(self.g_scrolledwindow)
-        self.g_scrolledwindow.add(self.g_textview)
-        vbox.pack_start(gtk.HSeparator(), False, False)
-        bbox = gtk.HButtonBox()
-        bbox.set_layout(gtk.BUTTONBOX_END)
-        vbox.pack_start(bbox, False, False)
-        self.g_close_button = gtk.Button(stock="gtk-close")
-        self.g_close_button.set_sensitive(False)
-        self.g_close_button.connect('clicked', lambda w: self.destroy())
-        bbox.pack_start(self.g_close_button)
-        self.show_all()
-    def run_finished(self):
-        self.g_close_button.set_sensitive(True)
+        self.add(self.g_textview)
     def write(self, s, tag=None):
         if tag:
             self.g_textbuffer.insert_with_tags_by_name(
@@ -843,6 +863,55 @@ class LogWindow(gtk.Window):
         # This is needed to make the window update:
         while gtk.events_pending():
             gtk.main_iteration()
+    def popen(self, *args, **kwargs):
+        """
+        Return the Popen objects .returncode
+        """
+        self.write("$ %s\n" % " ".join(args[0]))
+        p = osutils.Popen(
+            stdout=subprocess.PIPE,
+            stderr=subprocess.STDOUT,
+            *args, **kwargs)
+        while 1:
+            p.poll()
+            # returncode != None means that the process has finished
+            if p.returncode != None:
+                break
+            while 1:
+                s = p.stdout.readline()
+                if not s:
+                    break
+                self.write(s)
+            time.sleep(1)
+        p.wait()
+        return p.returncode
+
+
+class LogWindow(gtk.Window):
+    def __init__(self, parent):
+        gtk.Window.__init__(self)
+        self.set_transient_for(parent)
+        self.set_destroy_with_parent(True)
+        self.set_modal(True)
+        vbox = gtk.VBox()
+        vbox.set_spacing(4)
+        self.add(vbox)
+        self.set_default_size(630, 400)
+        self.g_logwidget = LogWidget()
+        vbox.pack_start(self.g_logwidget)
+        vbox.pack_start(gtk.HSeparator(), False, False)
+        bbox = gtk.HButtonBox()
+        bbox.set_layout(gtk.BUTTONBOX_END)
+        vbox.pack_start(bbox, False, False)
+        self.g_close_button = gtk.Button(stock="gtk-close")
+        self.g_close_button.set_sensitive(False)
+        self.g_close_button.connect('clicked', lambda w: self.destroy())
+        bbox.pack_start(self.g_close_button)
+        self.show_all()
+        self.write = self.g_logwidget.write
+        self.popen = self.g_logwidget.popen
+    def run_finished(self):
+        self.g_close_button.set_sensitive(True)
 
 
 class ClickableLabel(gtk.LinkButton):
@@ -853,8 +922,148 @@ class ClickableLabel(gtk.LinkButton):
         """
         gtk.LinkButton.__init__(self, label, label)
         self.get_children()[0].set_alignment(0.0, 0.5)
+    def add_heading(self, text):
+        def set_alignment(x, y):
+            for c in self.get_children()[0].get_children():
+                super(gtk.Label, c).set_alignment(x, y)
+        vbox = gtk.VBox()
+        vbox.set_alignment = set_alignment
+        b = gtk.Label()
+        b.set_markup(u"%s" % text)
+        b.set_alignment(0.0, 0.5)
+        vbox.pack_start(b, False)
+        self.get_children()[0].reparent(vbox)
+        self.add(vbox)
     def make_warning(self):
         im = gtk.Image()
         im.set_from_stock(gtk.STOCK_DIALOG_WARNING, gtk.ICON_SIZE_MENU)
         self.set_image(im)
         self.set_alignment(0.0, 0.5)
+
+
+class ExceptionDialog(gtk.Dialog):
+    def __init__(self, exception):
+        gtk.Dialog.__init__(self, sys.exc_info()[0].__name__)
+        self.set_resizable(True)
+        self.set_border_width(6)
+        self.set_has_separator(False)
+        self.vbox.set_spacing(0)
+        hbox = gtk.HBox()
+        hbox.set_spacing(12)
+        hbox.set_border_width(6)
+        self.vbox.pack_start(hbox, False)
+        vbox = gtk.VBox()
+        hbox.pack_start(vbox, False, False)
+        img = gtk.image_new_from_stock(gtk.STOCK_DIALOG_ERROR, gtk.ICON_SIZE_DIALOG)
+        vbox.pack_start(img, False, False)
+        vbox = gtk.VBox()
+        hbox.pack_start(vbox)
+        self.msg_vbox = gtk.VBox()
+        vbox.pack_start(self.msg_vbox)
+        self.g_primary = gtk.Label()
+        if sys.exc_info()[0].__name__ == 'AttributeError':
+            self.g_primary.set_name("DEBUGWARNING")
+        self.g_primary.set_alignment(0.0, 0.5)
+        self.g_primary.set_line_wrap(True)
+        self.m_primary_bold = False
+        self.msg_vbox.pack_start(self.g_primary)
+        if isinstance(exception, StandardError):
+            self.g_primary.set_text(str(exception).decode(sys.getfilesystemencoding(), 'replace'))
+        else:
+            if 'args' in dir(exception) and exception.args:
+                estr = exception.args[0]
+            else:
+                estr = exception
+            if not isinstance(estr, unicode):
+                estr = str(estr).decode(sys.getfilesystemencoding(), 'replace')
+            self.g_primary.set_text(estr)
+            if 'args' in dir(exception):
+                for s in exception.args[1:]:
+                    self.add_text(s)
+        # This label is here just for spacing
+        l = gtk.Label("")
+        vbox.pack_start(l)
+        expander = gtk.Expander("Traceback")
+        self.vbox.pack_start(expander)
+        l = gtk.Label("".join(traceback.format_exception(
+            sys.exc_type, sys.exc_value, sys.exc_traceback)).decode(sys.getfilesystemencoding(), 'replace'))
+        l.set_alignment(0.0, 0.5)
+        sc = gtk.ScrolledWindow()
+        sc.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
+        sc.set_size_request(-1, 100)
+        expander.add(sc)
+        sc.add_with_viewport(l)
+        l = gtk.Label(_("Visit http://www.solfege.org/support if you need help."))
+        l.set_alignment(0.0, 0.5)
+        vbox.pack_start(l, False)
+        w = self.add_button(gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE)
+        self.set_default_response(gtk.RESPONSE_CLOSE)
+        self.set_focus(w)
+        self.show_all()
+    def _make_primary_bold(self):
+        if not self.m_primary_bold:
+            self.m_primary_bold = True
+            self.g_primary.set_markup('<span weight="bold" size="larger">%s</span>' %
+                escape(self.g_primary.get_text()))
+    def _parsep(self):
+        l = gtk.Label("")
+        l.show()
+        self.msg_vbox.pack_start(l)
+    def add_text(self, text):
+        self._make_primary_bold()
+        self._parsep()
+        # We add a empty string with a newline to get the spacing
+        l = gtk.Label(text)
+        l.set_line_wrap(True)
+        l.set_alignment(0.0, 0.5)
+        l.show()
+        self.msg_vbox.pack_start(l)
+    def add_nonwrapped_text(self, text):
+        self._make_primary_bold()
+        self._parsep()
+        sc = gtk.ScrolledWindow()
+        l = gtk.Label()
+        l.set_markup('<span font_family="monospace">%s</span>' % escape(text))
+        l.set_line_wrap(False)
+        l.set_alignment(0.0, 0.5)
+        l.show()
+        sc.add_with_viewport(l)
+        self.msg_vbox.pack_start(sc)
+        sc.show_all()
+        sc.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_NEVER)
+        w, h = l.get_ancestor(gtk.Viewport).size_request()
+        max_lines = 5
+        lines = text.count("\n") + 1
+        # This is the space the horizontal scrollbar takes
+        scrollbar_height = (sc.get_hscrollbar().size_request()[1]
+            + sc.style_get_property("scrollbar-spacing"))
+        if lines <= max_lines:
+            # make the scrollwin so high that it can display all lines
+            sc.set_size_request(-1, h)
+        else:
+            sc.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
+            sc.set_size_request(-1, int(h * max_lines / lines))
+        adj = sc.get_hscrollbar().get_adjustment()
+        def f(scrollbar, event):
+            sc.set_size_request(-1, int(h * max_lines / lines) + scrollbar_height)
+            scrollbar.disconnect(self._hscroll_expose_id)
+        # f is called if the horizontal scrollbar is visible. This because we
+        # need to allocate space for the scrollbar too.
+        self._hscroll_expose_id = sc.get_hscrollbar().connect('expose-event', f)
+
+def display_exception_message(exception, lessonfile=None):
+        """Call this function only inside an except clause."""
+        sourcefile, lineno, func, code = traceback.extract_tb(sys.exc_info()[2])[0]
+        # We can replace characters because we will only display the
+        # file name, not open the file.
+        sourcefile = sourcefile.decode(sys.getfilesystemencoding(), 'replace')
+        m = ExceptionDialog(exception)
+        if lessonfile:
+            m.add_text(_("Please check the lesson file %s.") % lessonfile)
+        if sourcefile:
+            m.add_text(_('The exception was caught in\n"%(filename)s", line %(lineno)i.') % {'filename': sourcefile, 'lineno': lineno})
+        if 'm_nonwrapped_text' in dir(exception):
+            m.add_nonwrapped_text(exception.m_nonwrapped_text)
+        m.run()
+        m.destroy()
+
