@@ -1,6 +1,6 @@
 # -*- coding: iso-8859-1 -*-
 # GNU Solfege - free ear training software
-# Copyright (C) 2004, 2005, 2006, 2007, 2008  Tom Cato Amundsen
+# Copyright (C) 2004, 2005, 2006, 2007, 2008, 2011  Tom Cato Amundsen
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -29,7 +29,6 @@ from solfege import mpd
 from solfege import gu
 from solfege import lessonfile
 from solfege import dataparser
-from solfege import htmlwidget
 from solfege import stock
 
 gtk.stock_add([('solfege-notehead', _("Add noteheads"), 0, 0, ''),
@@ -208,7 +207,7 @@ class MainWin(gtk.Window):
         ## Header section
         sizegroup = gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)
         self.g_title = gtk.Entry()
-        cvbox.pack_start(gu.hig_label_widget(_("File title:"), self.g_title, 
+        cvbox.pack_start(gu.hig_label_widget(_("File title:"), self.g_title,
                         sizegroup))
         self.g_content_chord = gtk.RadioButton(None, "chord")
         self.g_content_chord_voicing = gtk.RadioButton(self.g_content_chord, "chord-voicing")
@@ -219,7 +218,7 @@ class MainWin(gtk.Window):
         box.pack_start(self.g_content_idbyname)
         cvbox.pack_start(gu.hig_label_widget(_("Content:"), box, sizegroup))
         self.g_random_transpose = gtk.Entry()
-        cvbox.pack_start(gu.hig_label_widget(_("Random transpose:"), 
+        cvbox.pack_start(gu.hig_label_widget(_("Random transpose:"),
             self.g_random_transpose, sizegroup))
         #
         #self.g_statusbar = gtk.Statusbar()
@@ -329,7 +328,7 @@ class MainWin(gtk.Window):
             self.save_file()
     def save_file(self):
         if not self.m_P.m_filename:
-            raise "No filename. Cannot save."
+            raise Exception("No filename. Cannot save.")
         ofile = open(self.m_P.m_filename, 'w')
         ofile.write("# Creator: GNU Solfege lesson file editor %s\n\n"
                     % app_version)
